/***************************************************************************
	file:			eeprom.c
	product:	API-004
	descr:		Diese API ist mit stack2Learn I2C-006 1k EEPROM und
						I2C-007 64k EEPROM kompatibel.
						
						Mehr Infos findes Du in eeprom.h und auf stack2Learn.de

	date: 		10.2013 - first release

	version 	1.0
	compiler:	Keil Cx51 V9.05

	mc:				AT89C5131A
	crystal:	24MHz

	author: 	Viktor Schabelski info@approbe.de
	licence:	Creative Commons Lizenz
                http://www.stack2Learn.de/cc.html

**************************************************/

/*-----------------------------------------------------------------------------
	includes
---------------------------------------------------*/
#include "eeprom.h"
#include "eeprom_config.h"
#include "../api-002_emuI2C/emuI2C.h"



/*-----------------------------------------------------------------------------
	prototype private functions
---------------------------------------------------*/



/*-----------------------------------------------------------------------------
	global variables
---------------------------------------------------*/



/*-----------------------------------------------------------------------------
	api
---------------------------------------------------*/
/*=====================================================================*//**
*	@brief		Diese Funktion schreibt ein Byte (8-bit) via I2C in das EEPROM.
*
*		An diese Funktion wird eine Adresse, unter der das Byte im EEPROM gespeichert werden soll
*		und eine 8-bit Variable uebergeben.
*
*		Die Variable "adress" teil der Funktion die Adresse im EEPPROM mit.
*		In "dataByte" steht das Byte (8-bit), das ins EEPROM geschrieben werden muss.
*
*		Diese Funktion verwendet folgende globale Konstanten aus der "eeprom_config.h" Datei:
*				- EEPROM_SLAVE_ADDRESS			Slave Adresse des Bausteins
*				- EEPROM_NUMBER_OF_ADDRESS	Anzahl der Adressen in EEPROM
*
*	@param		address - Adresse im EEPROM
*	@param		dataByte - 8-bit Varaible fuer EEPROM
*	@return		EEPROM_ERROR
*	@return		EEPROM_WRITE_FAIL
*	@return		EPPROM_SUCCESS
************************************************************************/
errorCodeEEPROM_t EEPROM_WriteByte(unsigned int address, unsigned char dataByte){
	errorCodeEEPROM_t error = EEPROM_BUSY;
	unsigned char dataToSend[4] = {0x00};
	// Anzahl von Bytes zum Senden via I2C
	unsigned char sizeOfDataToSend = 0;

	// Ueberpruefung der hoechsten Adresse
	if(address > EEPROM_NUMBER_OF_ADDRESS){
		return (EEPROM_ERROR);
	}
	
	// Daten zum Senden vorbereiten
	dataToSend[0] = EEPROM_SLAVE_ADDRESS;	
	
#if EEPROM_MODUL == I2C_006_EEPROM
	dataToSend[1] = (unsigned char)address;
	dataToSend[2] = dataByte;
	sizeOfDataToSend = 3;
	
#elif EEPROM_MODUL == I2C_007_EEPROM
	dataToSend[1] = ((unsigned char)(address >> 8));
	dataToSend[2] = (unsigned char)address;
	dataToSend[3] = dataByte;
	sizeOfDataToSend = 4;
#endif

	// sende Bytes
	error = EmuI2C_SendData(dataToSend, sizeOfDataToSend);

	if (error == EMU_I2C_SUCCESS){
		return (EEPROM_SUCCESS);
	}else{
		return (EEPROM_WRITE_FAIL);
	}
}


/*=====================================================================*//**
*	@brief		Diese Funktion liest einen Byte (8-bit) via I2C aus dem EEPROM.
*
*		An diese Funktion wird eine Adresse, unter der das Byte im EEPROM gespeichert ist
*		und ein Pointer auf eine 8-bit Variable uebergeben.
*
*		Die Variable "adress" teil der Funktion die Adresse im EEPPROM mit.
*		In die Variable "dataByte" wird das gelesene Byte (8-bit) geschrieben.
*
*		Diese Funktion verwendet folgende globale Konstanten aus der "eeprom_config.h" Datei:
*				- EEPROM_MODUL und I2C_006_EEPROM		Definition von verwendetem Modul
*				- EEPROM_SLAVE_ADDRESS			Slave Adresse des Bausteins
*				- EEPROM_NUMBER_OF_ADDRESS	Anzahl der Adressen in EEPROM
*
*	@param		address - Adresse im EEPROM
*	@param		dataByte - 8-bit Varaible fuer EEPROM
*	@return		EEPROM_ERROR
*	@return		EEPROM_READ_FAIL
*	@return		EPPROM_SUCCESS
************************************************************************/
errorCodeEEPROM_t EEPROM_ReadByte(unsigned int address, unsigned char* dataByte){
	errorCodeEEPROM_t error = EEPROM_BUSY;
	unsigned char dataToSend[3] = {0x00};
	unsigned char sizeOfDataToSend = 0, sizeOfDataToReceive = 0;
	
	
	// Daten zum Senden vorbereiten
	dataToSend[0] = EEPROM_SLAVE_ADDRESS;	
	
#if EEPROM_MODUL == I2C_006_EEPROM
	dataToSend[1] = (unsigned char)address;
	sizeOfDataToSend = 2;
	
#elif EEPROM_MODUL == I2C_007_EEPROM
	dataToSend[1] = ((unsigned char)(address >> 8));
	dataToSend[2] = (unsigned char)address;
	sizeOfDataToSend = 3;
#endif

	// sende Bytes
	error = EmuI2C_SendData(dataToSend, sizeOfDataToSend);
	if (error != EMU_I2C_SUCCESS){
				return (EEPROM_READ_FAIL);
	}else{
	}

	// Empfange Daten
	// Daten zum Senden vorbereiten
	sizeOfDataToReceive = 1;
	dataByte [0] = (EEPROM_SLAVE_ADDRESS + 1);
	error = EmuI2C_ReceiveData(dataByte, sizeOfDataToReceive);
	if (error == EMU_I2C_SUCCESS){
		return (EEPROM_SUCCESS);
	}else{
		return (EEPROM_READ_FAIL);
	}
}